@@ -30,7 +30,7 @@ from kernels.smm_acc_predict import (  # noqa: E402
 def main(
     param_fn,
     compiler,
-    cpus_per_node,
+    cpus_per_task,
     max_num_nodes,
     blocksizes,
     blocks_from_param_file,
@@ -86,7 +86,7 @@ def main(
             continue
         os.mkdir(outdir)
         gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n, k)
-        gen_jobfile(outdir, compiler, m, n, k, cpus_per_node, max_num_nodes)
+        gen_jobfile(outdir, compiler, m, n, k, cpus_per_task, max_num_nodes)
         gen_makefile(outdir, compiler, arch_code)
 
 
@@ -214,7 +214,7 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
             )
 
         output += "\n"
-        output += "int main(int argc, char** argv){\n"
+        output += "int main(int argc, char** argv) {\n"
         if compiler == "nvcc":
             output += (
                 indent
@@ -256,7 +256,7 @@ def gen_benchmark(outdir, gpu_properties, autotuning_properties, compiler, m, n,
 
 
 # ===============================================================================
-def gen_jobfile(outdir, compiler, m, n, k, cpus_per_node=12, max_num_nodes=0):
+def gen_jobfile(outdir, compiler, m, n, k, cpus_per_task=12, max_num_nodes=0):
 
     file_extension = get_file_extension_from_compiler(compiler)
 
@@ -278,7 +278,7 @@ def gen_jobfile(outdir, compiler, m, n, k, cpus_per_node=12, max_num_nodes=0):
     output += "#SBATCH --nodes=%d\n" % num_nodes
     output += "#SBATCH --ntasks-per-core=1\n"
     output += "#SBATCH --ntasks-per-node=1\n"
-    output += "#SBATCH --cpus-per-task=" + "%d\n" % cpus_per_node
+    output += "#SBATCH --cpus-per-task=" + "%d\n" % cpus_per_task
     output += "#SBATCH --time=%s\n" % time
     output += "#SBATCH --partition=normal\n"
     output += "#SBATCH --constraint=gpu\n"
@@ -302,7 +302,7 @@ def gen_jobfile(outdir, compiler, m, n, k, cpus_per_node=12, max_num_nodes=0):
     for exe in all_exe:
         output += (
             "srun --nodes=1 --bcast=/tmp/${USER} --ntasks=1 --ntasks-per-node=1 --cpus-per-task=%d make -j %d %s &\n"
-            % (cpus_per_node, 2 * cpus_per_node, exe)
+            % (cpus_per_task, 2 * cpus_per_task, exe)
         )
         num_nodes_busy += 1
         if num_nodes_busy == num_nodes:
@@ -406,9 +406,8 @@ def gen_makefile(outdir, compiler, arch):
                 + " -w -o $@ $^ -lcuda -lnvrtc\n\n"
             )
         else:
-            output += (
-                "\thipcc -O3 -D__HIP -w -o $@ $^ /opt/rocm/hip/lib/libhiprtc.so\n\n"
-            )
+            rocm_path = os.getenv("ROCM_PATH", "/opt/rocm")
+            output += f"\thipcc -O3 -D__HIP -w -o $@ $^ {rocm_path}/hip/lib/libamdhip64.so\n\n"
 
     # write Makefile
     writefile(outdir + "/Makefile", output)
@@ -455,7 +454,7 @@ if __name__ == "__main__":
         containing the code, Makefile and jobfiles for the autotuning of a given (m, n, k)-triplet.
 
         This script is part of the workflow for autotuning optimal libsmm_acc parameters.
-        For more details, see README.md#autotuning-procedure.
+        For more details, see README.md.
         """,
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
     )
@@ -479,15 +478,15 @@ if __name__ == "__main__":
         "--compiler",
         metavar="compiler",
         default="nvcc",
-        help="Compiler to use for compiling kernel code (Opions: nvcc, hipcc)",
+        help="Compiler to use for compiling kernel code (Options: nvcc, hipcc)",
     )
     parser.add_argument(
         "-c",
-        "--cpus_per_node",
+        "--cpus_per_task",
         metavar="INT",
         default=12,
         type=int,
-        help="Number of CPUs per node",
+        help="Number of CPUs required per task",
     )
     parser.add_argument(
         "-n",
@@ -532,7 +531,7 @@ if __name__ == "__main__":
     main(
         args.params,
         args.compiler,
-        args.cpus_per_node,
+        args.cpus_per_task,
         args.nodes,
         args.blocksizes,
         blocksizes_from_param_file,
