@@ -1,8 +1,8 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2020 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
-/*  SPDX-License-Identifier: GPL-2.0-or-later                                 */
+/*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
 #ifndef GRID_CONSTANTS_H
 #define GRID_CONSTANTS_H
@@ -33,7 +33,16 @@ enum grid_func {
   GRID_FUNC_DZDX = 703,
   GRID_FUNC_DXDX = 801,
   GRID_FUNC_DYDY = 802,
-  GRID_FUNC_DZDZ = 803
+  GRID_FUNC_DZDZ = 803,
+  GRID_FUNC_DAB_X = 901,
+  GRID_FUNC_DAB_Y = 902,
+  GRID_FUNC_DAB_Z = 903,
+  GRID_FUNC_ADB_X = 904,
+  GRID_FUNC_ADB_Y = 905,
+  GRID_FUNC_ADB_Z = 906,
+  GRID_FUNC_CORE_X = 1001,
+  GRID_FUNC_CORE_Y = 1002,
+  GRID_FUNC_CORE_Z = 1003,
 };
 
 enum grid_backend {
@@ -41,7 +50,6 @@ enum grid_backend {
   GRID_BACKEND_REF = 11,
   GRID_BACKEND_CPU = 12,
   GRID_BACKEND_GPU = 13,
-  GRID_BACKEND_HYBRID = 14
 };
 
 #endif
