@@ -1,8 +1,8 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2020 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
-/*  SPDX-License-Identifier: GPL-2.0-or-later                                 */
+/*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
 #ifndef GRID_GPU_COLLOCATE_H
 #define GRID_GPU_COLLOCATE_H
@@ -22,11 +22,9 @@ extern "C" {
  ******************************************************************************/
 void grid_gpu_collocate_one_grid_level(
     const grid_gpu_task_list *task_list, const int first_task,
-    const int last_task, const bool orthorhombic, const enum grid_func func,
-    const int npts_global[3], const int npts_local[3], const int shift_local[3],
-    const int border_width[3], const double dh[3][3], const double dh_inv[3][3],
-    const cudaStream_t stream, const double *pab_blocks_dev, double *grid_dev,
-    int *lp_diff);
+    const int last_task, const enum grid_func func,
+    const grid_gpu_layout *layout, const cudaStream_t stream,
+    const double *pab_blocks_dev, double *grid_dev, int *lp_diff);
 
 #ifdef __cplusplus
 }
