@@ -1,12 +1,12 @@
 /*----------------------------------------------------------------------------*/
 /*  CP2K: A general program to perform molecular dynamics simulations         */
-/*  Copyright 2000-2020 CP2K developers group <https://cp2k.org>              */
+/*  Copyright 2000-2021 CP2K developers group <https://cp2k.org>              */
 /*                                                                            */
-/*  SPDX-License-Identifier: GPL-2.0-or-later                                 */
+/*  SPDX-License-Identifier: BSD-3-Clause                                     */
 /*----------------------------------------------------------------------------*/
 
 // needed for struct timespec
-#define _XOPEN_SOURCE 700 /* Enable POSIX 2008/13 */
+#define _POSIX_C_SOURCE 200809L
 
 #include <assert.h>
 #include <fenv.h>
@@ -19,9 +19,10 @@
 #include <string.h>
 #include <time.h>
 
-#include "common/grid_buffer.h"
+#include "../offload/offload_buffer.h"
 #include "common/grid_common.h"
 #include "grid_replay.h"
+
 #include "grid_task_list.h"
 #include "ref/grid_ref_collocate.h"
 #include "ref/grid_ref_integrate.h"
@@ -154,20 +155,27 @@ static void create_dummy_basis_set(const int size, const int lmin,
  * \brief Creates mock task list with one task per cycle.
  * \author Ole Schuett
  ******************************************************************************/
-static void create_dummy_task_list(const int border_mask, const double ra[3],
-                                   const double rab[3], const double radius,
-                                   const grid_basis_set *basis_set_a,
-                                   const grid_basis_set *basis_set_b,
-                                   const int o1, const int o2, const int la_max,
-                                   const int lb_max, const int cycles,
-                                   const int cycles_per_block,
-                                   grid_task_list **task_list) {
+static void create_dummy_task_list(
+    const bool orthorhombic, const int border_mask, const double ra[3],
+    const double rab[3], const double radius, const grid_basis_set *basis_set_a,
+    const grid_basis_set *basis_set_b, const int o1, const int o2,
+    const int la_max, const int lb_max, const int cycles,
+    const int cycles_per_block, const int npts_global[][3],
+    const int npts_local[][3], const int shift_local[][3],
+    const int border_width[][3], const double dh[][3][3],
+    const double dh_inv[][3][3], grid_task_list **task_list) {
 
   const int ntasks = cycles;
   const int nlevels = 1;
   const int natoms = 2;
   const int nkinds = 2;
-  const int nblocks = cycles / cycles_per_block + 1;
+  int nblocks = cycles / cycles_per_block + 1;
+
+  /* we can not have more blocks than the number of tasks */
+  if (cycles == 1) {
+    nblocks = 1;
+  }
+
   int block_offsets[nblocks];
   memset(block_offsets, 0, nblocks * sizeof(int)); // all point to same data
   const double atom_positions[2][3] = {
@@ -201,11 +209,12 @@ static void create_dummy_task_list(const int border_mask, const double ra[3],
   }
   const double(*rab_list)[3] = (const double(*)[3])rab_list_mutable;
 
-  grid_create_task_list(ntasks, nlevels, natoms, nkinds, nblocks, block_offsets,
-                        atom_positions, atom_kinds, basis_sets, level_list,
-                        iatom_list, jatom_list, iset_list, jset_list, ipgf_list,
-                        jpgf_list, border_mask_list, block_num_list,
-                        radius_list, rab_list, task_list);
+  grid_create_task_list(
+      orthorhombic, ntasks, nlevels, natoms, nkinds, nblocks, block_offsets,
+      atom_positions, atom_kinds, basis_sets, level_list, iatom_list,
+      jatom_list, iset_list, jset_list, ipgf_list, jpgf_list, border_mask_list,
+      block_num_list, radius_list, rab_list, npts_global, npts_local,
+      shift_local, border_width, dh, dh_inv, task_list);
 }
 
 /*******************************************************************************
@@ -242,7 +251,7 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
 
   const bool orthorhombic = parse_int("orthorhombic", fp);
   const int border_mask = parse_int("border_mask", fp);
-  const int func = parse_int("func", fp);
+  const enum grid_func func = (enum grid_func)parse_int("func", fp);
   const bool compute_tau = (func == GRID_FUNC_DADB);
   const int la_max = parse_int("la_max", fp);
   const int la_min = parse_int("la_min", fp);
@@ -283,16 +292,17 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
   const double(*pab)[n1] = (const double(*)[n1])pab_mutable;
 
   const int npts_local_total = npts_local[0] * npts_local[1] * npts_local[2];
-  const size_t sizeof_grid = sizeof(double) * npts_local_total;
-  double *grid_ref = malloc(sizeof_grid);
-  memset(grid_ref, 0, sizeof_grid);
+  offload_buffer *grid_ref = NULL;
+  offload_create_buffer(npts_local_total, &grid_ref);
+  memset(grid_ref->host_buffer, 0, npts_local_total * sizeof(double));
 
   const int ngrid_nonzero = parse_int("ngrid_nonzero", fp);
   for (int n = 0; n < ngrid_nonzero; n++) {
     int i, j, k;
     double value;
     parse_next_line("grid", fp, "%i %i %i %le", 4, &i, &j, &k, &value);
-    grid_ref[k * npts_local[1] * npts_local[0] + j * npts_local[0] + i] = value;
+    grid_ref->host_buffer[k * npts_local[1] * npts_local[0] +
+                          j * npts_local[0] + i] = value;
   }
 
   double hab_ref[n2][n1];
@@ -318,7 +328,8 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
     abort();
   }
 
-  double *grid_test = malloc(sizeof_grid);
+  offload_buffer *grid_test = NULL;
+  offload_create_buffer(npts_local_total, &grid_test);
   double hab_test[n2][n1];
   double forces_test[2][3];
   double virial_test[3][3];
@@ -330,12 +341,15 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
     create_dummy_basis_set(n1, la_min, la_max, zeta, &basisa);
     create_dummy_basis_set(n2, lb_min, lb_max, zetb, &basisb);
     grid_task_list *task_list = NULL;
-    create_dummy_task_list(border_mask, ra, rab, radius, basisa, basisb, o1, o2,
-                           la_max, lb_max, cycles, cycles_per_block,
-                           &task_list);
-    grid_buffer *pab_blocks = NULL, *hab_blocks = NULL;
-    grid_create_buffer(n1 * n2, &pab_blocks);
-    grid_create_buffer(n1 * n2, &hab_blocks);
+    create_dummy_task_list(
+        orthorhombic, border_mask, ra, rab, radius, basisa, basisb, o1, o2,
+        la_max, lb_max, cycles, cycles_per_block, (const int(*)[3])npts_global,
+        (const int(*)[3])npts_local, (const int(*)[3])shift_local,
+        (const int(*)[3])border_width, (const double(*)[3][3])dh,
+        (const double(*)[3][3])dh_inv, &task_list);
+    offload_buffer *pab_blocks = NULL, *hab_blocks = NULL;
+    offload_create_buffer(n1 * n2, &pab_blocks);
+    offload_create_buffer(n1 * n2, &hab_blocks);
     const double f = (collocate) ? rscale : 1.0;
     for (int i = 0; i < n1; i++) {
       for (int j = 0; j < n2; j++) {
@@ -347,21 +361,15 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
     const int natoms = 2;
     if (collocate) {
       // collocate
-      double *grids_array[1] = {grid_test};
-      grid_collocate_task_list(
-          task_list, orthorhombic, func, nlevels, (const int(*)[3])npts_global,
-          (const int(*)[3])npts_local, (const int(*)[3])shift_local,
-          (const int(*)[3])border_width, (const double(*)[3][3])dh,
-          (const double(*)[3][3])dh_inv, pab_blocks, grids_array);
+      offload_buffer *grids[1] = {grid_test};
+      grid_collocate_task_list(task_list, func, nlevels,
+                               (const int(*)[3])npts_local, pab_blocks, grids);
     } else {
       // integrate
-      double *grids_array[1] = {grid_ref};
-      grid_integrate_task_list(
-          task_list, orthorhombic, compute_tau, natoms, nlevels,
-          (const int(*)[3])npts_global, (const int(*)[3])npts_local,
-          (const int(*)[3])shift_local, (const int(*)[3])border_width,
-          (const double(*)[3][3])dh, (const double(*)[3][3])dh_inv, pab_blocks,
-          (const double(**))grids_array, hab_blocks, forces_test, virial_test);
+      const offload_buffer *grids[1] = {grid_ref};
+      grid_integrate_task_list(task_list, compute_tau, natoms, nlevels,
+                               (const int(*)[3])npts_local, pab_blocks, grids,
+                               hab_blocks, forces_test, virial_test);
       for (int i = 0; i < n2; i++) {
         for (int j = 0; j < n1; j++) {
           hab_test[i][j] = hab_blocks->host_buffer[i * n1 + j];
@@ -372,18 +380,19 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
     grid_free_basis_set(basisa);
     grid_free_basis_set(basisb);
     grid_free_task_list(task_list);
-    grid_free_buffer(pab_blocks);
-    grid_free_buffer(hab_blocks);
+    offload_free_buffer(pab_blocks);
+    offload_free_buffer(hab_blocks);
   } else {
     clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &start_time);
     if (collocate) {
       // collocate
-      memset(grid_test, 0, sizeof_grid);
+      memset(grid_test->host_buffer, 0, npts_local_total * sizeof(double));
       for (int i = 0; i < cycles; i++) {
         grid_ref_collocate_pgf_product(
             orthorhombic, border_mask, func, la_max, la_min, lb_max, lb_min,
             zeta, zetb, rscale, dh, dh_inv, ra, rab, npts_global, npts_local,
-            shift_local, border_width, radius, o1, o2, n1, n2, pab, grid_test);
+            shift_local, border_width, radius, o1, o2, n1, n2, pab,
+            grid_test->host_buffer);
       }
     } else {
       // integrate
@@ -394,8 +403,9 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
         grid_ref_integrate_pgf_product(
             orthorhombic, compute_tau, border_mask, la_max, la_min, lb_max,
             lb_min, zeta, zetb, dh, dh_inv, ra, rab, npts_global, npts_local,
-            shift_local, border_width, radius, o1, o2, n1, n2, grid_ref,
-            hab_test, pab, forces_test, virials_test, NULL, NULL);
+            shift_local, border_width, radius, o1, o2, n1, n2,
+            grid_ref->host_buffer, hab_test, pab, forces_test, virials_test,
+            NULL, NULL, NULL);
       }
       for (int i = 0; i < 3; i++) {
         for (int j = 0; j < 3; j++) {
@@ -414,8 +424,8 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
     // collocate
     // compare grid
     for (int i = 0; i < npts_local_total; i++) {
-      const double ref_value = cycles * grid_ref[i];
-      const double test_value = grid_test[i];
+      const double ref_value = cycles * grid_ref->host_buffer[i];
+      const double test_value = grid_test->host_buffer[i];
       const double diff = fabs(test_value - ref_value);
       const double rel_diff = diff / fmax(1.0, fabs(ref_value));
       max_rel_diff = fmax(max_rel_diff, rel_diff);
@@ -432,10 +442,10 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
         const double rel_diff = diff / fmax(1.0, fabs(ref_value));
         max_rel_diff = fmax(max_rel_diff, rel_diff);
         max_value = fmax(max_value, fabs(test_value));
-        // if (ref_value != 0.0 || test_value != 0.0) {
-        //   printf("%i %i ref: %le test: %le diff:%le rel_diff: %le\n", i, j,
-        //          ref_value, test_value, diff, rel_diff);
-        // }
+        if (max_rel_diff > 1e-14) {
+          printf("%i %i ref: %le test: %le diff:%le rel_diff: %le\n", i, j,
+                 ref_value, test_value, diff, rel_diff);
+        }
       }
     }
     // compare forces
@@ -447,6 +457,10 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
         const double diff = fabs(test_value - ref_value);
         const double rel_diff = diff / fmax(1.0, fabs(ref_value));
         max_rel_diff = fmax(max_rel_diff, rel_diff * forces_fudge_factor);
+        if (max_rel_diff > 1e-14) {
+          printf("forces %i %i ref: %le test: %le diff:%le rel_diff: %le\n", i,
+                 j, ref_value, test_value, diff, rel_diff);
+        }
       }
     }
     // compare virial
@@ -458,6 +472,10 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
         const double diff = fabs(test_value - ref_value);
         const double rel_diff = diff / fmax(1.0, fabs(ref_value));
         max_rel_diff = fmax(max_rel_diff, rel_diff * virial_fudge_factor);
+        if (max_rel_diff > 1e-14) {
+          printf("virial %i %i ref: %le test: %le diff:%le rel_diff: %le\n", i,
+                 j, ref_value, test_value, diff, rel_diff);
+        }
       }
     }
   }
@@ -467,8 +485,8 @@ double grid_replay(const char *filename, const int cycles, const bool collocate,
          batch ? "Batched" : "PGF-Ref", (float)cycles, max_value, max_rel_diff,
          delta_sec);
 
-  free(grid_ref);
-  free(grid_test);
+  offload_free_buffer(grid_ref);
+  offload_free_buffer(grid_test);
 
   // Check floating point exceptions.
   if (fetestexcept(FE_DIVBYZERO) != 0) {
