@@ -2,9 +2,9 @@
 
 # author: Ole Schuett
 
-if (( $# != 1 )) ; then
-    echo "Usage: test_coverage.sh <VERSION>"
-    exit 1
+if (($# != 1)); then
+  echo "Usage: test_coverage.sh <VERSION>"
+  exit 1
 fi
 
 ARCH=local_coverage
@@ -13,11 +13,23 @@ VERSION=$1
 # shellcheck disable=SC1091
 source /opt/cp2k-toolchain/install/setup
 
-echo -e "\n========== Running Regtests =========="
 cd /workspace/cp2k || exit 1
 CP2K_REVISION=$(./tools/build_utils/get_revision_number ./src)
-rm -rf "obj/${ARCH}/${VERSION}"/*.gcda   # remove old gcov statistics
+rm -rf "obj/${ARCH}/${VERSION}"/*.gcda # remove old gcov statistics
 
+# Compile cp2k.
+echo -en "\nCompiling cp2k... "
+if make -j ARCH="${ARCH}" VERSION="${VERSION}" &> make.out; then
+  echo "done."
+else
+  echo -e "failed.\n\n"
+  tail -n 100 make.out
+  echo -e "\nSummary: Compilation failed."
+  echo -e "Status: FAILED\n"
+  exit 0
+fi
+
+echo -e "\n========== Running Regtests =========="
 make ARCH="${ARCH}" VERSION="${VERSION}" TESTOPTS="${TESTOPTS}" test
 
 # gcov gets stuck on some files...
@@ -26,17 +38,20 @@ make ARCH="${ARCH}" VERSION="${VERSION}" TESTOPTS="${TESTOPTS}" test
 rm -f "/workspace/cp2k/obj/${ARCH}/${VERSION}"/exts/*/*.gcda
 cd /tmp || exit 1
 GCOV_TIMEOUT="10s"
-for fn in "/workspace/cp2k/obj/${ARCH}/${VERSION}"/*.gcda ; do
-    if ! timeout "${GCOV_TIMEOUT}" gcov "$fn" &> /dev/null ; then
-        echo "Skipping ${fn} because gcov took longer than ${GCOV_TIMEOUT}."
-        rm "${fn}"
-    fi
+for fn in "/workspace/cp2k/obj/${ARCH}/${VERSION}"/*.gcda; do
+  if ! timeout "${GCOV_TIMEOUT}" gcov "$fn" &> /dev/null; then
+    echo "Skipping ${fn} because gcov took longer than ${GCOV_TIMEOUT}."
+    rm "${fn}"
+  fi
 done
 
 # collect coverage stats
 mkdir -p /workspace/artifacts/coverage
 cd /workspace/artifacts/coverage || exit 1
-lcov --directory "/workspace/cp2k/obj/${ARCH}/${VERSION}" --capture --output-file coverage.info > lcov.log
+lcov --directory "/workspace/cp2k/obj/${ARCH}/${VERSION}" \
+  --exclude "/opt/cp2k-toolchain/*" \
+  --capture \
+  --output-file coverage.info > lcov.log
 lcov --summary coverage.info
 
 # generate html report
