@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 import sys, re
 
@@ -7,7 +7,7 @@ FYPP_LINE = r"^\s*" + FYPP_SYMBOLS + r":"
 FYPP_INLINE = r"(" + FYPP_SYMBOLS + r"{|}" + FYPP_SYMBOLS + r")"
 FYPP_RE = re.compile(r"(" + FYPP_LINE + r"|" + FYPP_INLINE + r")")
 
-with open(sys.argv[1], "r") as infile:
+with open(sys.argv[1], "r", encoding="utf8") as infile:
     if any(FYPP_RE.search(l) for l in infile):
         sys.exit(0)
 
