@@ -1,31 +1,84 @@
 import sys
 import re
-from collections import deque
 import logging
+from collections import deque
+from io import StringIO
 
-try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
+# Declare various RE snippets for building larger REs
 
-R_USE = 0
-R_VAR = 0
+# Valid fortran argument/variable name
+VALID_NAME = r"[a-zA-Z][a-zA-Z_0-9]*"
+
+# Set up valid type declaration
+DECL_KIND_KW = rf"\( (?:kind = )? (?:{VALID_NAME}|[0-9]+) \)".replace(" ", r"\s*")
+DECL_KIND_STAR = r"\s*\*\s*[0-9]+"
+CHAR_KIND_SPEC = rf"(?:(?:len|kind) = )? (?:{VALID_NAME}|[0-9]+|\*|:)"
+CHAR_KIND_SPEC = rf"\( {CHAR_KIND_SPEC}(?: , {CHAR_KIND_SPEC}) \)".replace(" ", r"\s*")
+DECL_KIND_SPEC = rf"\s*(?:{DECL_KIND_STAR}|{DECL_KIND_KW})"
+DECL_TYPE_DEF = rf"""(?:
+(?:integer|real|complex)(?:{DECL_KIND_SPEC})?|
+character(?:{DECL_KIND_STAR}|{CHAR_KIND_SPEC})?|
+logical(?:{DECL_KIND_KW})?|type|class)"""
+
+# String styles
+DQUOTED = r'"[^"]*"'
+SQUOTED = r"'[^']*'"
+QUOTED = rf"(?:{DQUOTED}|{SQUOTED})"
+
+# Stop at punctuation
+NOT_PUNC = r"[^()\"'\[\]]"
+NOT_PUNC_COMMA = r"[^()\"',\[\]]"
+NOT_BRAC = r"[^()]"
+
+# Enable new-style fortran arrays (square brackets)
+BRAC_OPEN = r"[(\[]"
+BRAC_CLOSE = r"[)\]]"
+NOT_PAREN = r"[^()]"
+
+# Nested parens for up to 3-level nesting (1st level's parens are outside this variable)
+# Should more than 3 levels be necessary, need to reduplicate this deeper.
+NESTED_PAREN_1TO3_CONTENTS = (
+    rf"(?:{NOT_PAREN}+|\((?:{NOT_PAREN}+|\({NOT_PAREN}*\))*\))*"
+)
+
+# Variable statement
 VAR_RE = re.compile(
-    r" *(?P<var>[a-zA-Z_0-9]+) *(?P<rest>(?:\((?P<param>(?:[^()]+|\((?:[^()]+|\([^()]*\))*\))*)\))? *(?:= *(?P<value>(:?[^\"',()]+|\((?:[^()\"']+|\([^()\"']*\)|\"[^\"]*\"|'[^']*')*\)|\"[^\"]*\"|'[^']*')+))?)? *(?:(?P<continue>,)|\n?) *",
-    re.IGNORECASE,
+    rf"""
+\s*(?P<var>{VALID_NAME})
+\s*(?P<rest>(:?\((?P<param>{NESTED_PAREN_1TO3_CONTENTS})\))?
+\s*(?:(?P<assignment>=>?)\s*(?P<value>(?:
+{NOT_PUNC_COMMA}+|
+{BRAC_OPEN}(?:{NOT_PUNC}|{BRAC_OPEN}{NOT_PUNC}*{BRAC_CLOSE}|{QUOTED})*{BRAC_CLOSE}|
+{QUOTED})+))?)?
+\s*(?:(?P<continue>,)|\n?)
+\s*""",
+    re.IGNORECASE | re.VERBOSE,
 )
+
+# Use statement
 USE_PARSE_RE = re.compile(
-    r" *use +(?P<module>[a-zA-Z_][a-zA-Z_0-9]*)(?P<only> *, *only *:)? *(?P<imports>.*)$",
-    flags=re.IGNORECASE,
+    rf"""\s*
+use(\s+|(?P<intrinsic>\s*,\s*Intrinsic\s+::\s*))
+(?P<module>{VALID_NAME})
+(?P<only>\s*,\s*only\s*:)?\s*
+(?P<imports>.*)$""",
+    flags=re.IGNORECASE | re.VERBOSE,
 )
-COMMON_USES_RE = re.compile('^#include *"([^"]*(cp_common_uses.f90|base_uses.f90))"')
+
+COMMON_USES_RE = re.compile('^#include\s*"([^"]*(cp_common_uses.f90|base_uses.f90))"')
 LOCAL_NAME_RE = re.compile(
-    " *(?P<localName>[a-zA-Z_0-9]+)(?: *= *> *[a-zA-Z_0-9]+)? *$"
+    rf"\s*(?P<localName>{VALID_NAME})(?:\s*=>\s*{VALID_NAME})?\s*$", re.VERBOSE
 )
 VAR_DECL_RE = re.compile(
-    r" *(?P<type>integer(?: *\* *[0-9]+)?|logical|character(?: *\* *[0-9]+)?|real(?: *\* *[0-9]+)?|complex(?: *\* *[0-9]+)?|type) *(?P<parameters>\((?:[^()]+|\((?:[^()]+|\([^()]*\))*\))*\))? *(?P<attributes>(?: *, *[a-zA-Z_0-9]+(?: *\((?:[^()]+|\((?:[^()]+|\([^()]*\))*\))*\))?)+)? *(?P<dpnt>::)?(?P<vars>[^\n]+)\n?",
-    re.IGNORECASE,
-)  # $
+    rf"""
+\s*(?P<type>{DECL_TYPE_DEF})
+\s*(?P<parameters>\({NESTED_PAREN_1TO3_CONTENTS}\))?
+\s*(?P<attributes>(?:\s*,\s*[a-zA-Z_0-9]+(?:\s*\({NESTED_PAREN_1TO3_CONTENTS}\))?)+)?\s*
+(?P<dpnt>::)?
+(?P<vars>[^\n]+)\n?
+""",
+    re.IGNORECASE | re.VERBOSE,
+)
 INDENT_SIZE = 2
 DECL_LINELENGTH = 100
 DECL_OFFSET = 50
@@ -67,11 +120,6 @@ NON_WORD_RE = re.compile(r"(\(/|/\)|[^-+a-zA-Z0-9_.])")
 
 STR_RE = re.compile(r"('[^'\n]*'|\"[^\"\n]*\")")
 
-COMMENT_TO_REMOVE_RE = re.compile(
-    r" *! *(?:interface|arguments|parameters|locals?|\** *local +variables *\**|\** *local +parameters *\**) *$",
-    re.IGNORECASE,
-)
-
 MODULE_N_RE = re.compile(
     r".*:: *moduleN *= *(['\"])[a-zA-Z_0-9]+\1", flags=re.IGNORECASE
 )
@@ -161,7 +209,7 @@ class InputStream(object):
         lines = []
         continuation = 0
 
-        while 1:
+        while True:
             if not self.line_buffer:
                 line = self.infile.readline().replace("\t", 8 * " ")
                 self.line_nr += 1
@@ -231,9 +279,8 @@ class InputStream(object):
         return (joinedLine, comments, lines)
 
 
-def parseRoutine(inFile):
+def parseRoutine(inFile, logger):
     """Parses a routine"""
-    logger = logging.getLogger("prettify-logger")
 
     routine = {
         "preRoutine": [],
@@ -255,7 +302,7 @@ def parseRoutine(inFile):
         "use": [],
     }
     stream = InputStream(inFile)
-    while 1:
+    while True:
         (jline, _, lines) = stream.nextFortranLine()
         if len(lines) == 0:
             break
@@ -265,9 +312,9 @@ def parseRoutine(inFile):
         m = INCLUDE_RE.match(lines[0])
         if m:
             try:
-                subF = open(m.group("file"), "r")
+                subF = open(m.group("file"), "r", encoding="utf8")
                 subStream = InputStream(subF)
-                while 1:
+                while True:
                     (subjline, _, sublines) = subStream.nextFortranLine()
                     if not sublines:
                         break
@@ -276,12 +323,15 @@ def parseRoutine(inFile):
             except:
                 import traceback
 
-                logger.debug("error trying to follow include " + m.group("file") + "\n")
                 logger.debug(
-                    "warning this might lead to the removal of used variables\n"
+                    "error trying to follow include '{}', this might lead to the removal of used variables".format(
+                        m.group("file")
+                    )
                 )
+
                 if logger.isEnabledFor(logging.DEBUG):
                     traceback.print_exc()
+
     if jline:
         routine["begin"] = lines
         m = START_ROUTINE_RE.match(jline)
@@ -297,7 +347,8 @@ def parseRoutine(inFile):
             routine["result"] = m.group("result")
         if (not routine["result"]) and (routine["kind"].lower() == "function"):
             routine["result"] = routine["name"]
-    while 1:
+
+    while True:
         (jline, comment_list, lines) = stream.nextFortranLine()
         comments = "\n".join(_ for _ in comment_list)
         if len(lines) == 0:
@@ -341,7 +392,7 @@ def parseRoutine(inFile):
                     )
                     str = str[m2.span()[1] :]
                 str = m.group("vars")
-                while 1:
+                while True:
                     m2 = VAR_RE.match(str)
                     if not m2:
                         raise SyntaxError(
@@ -351,7 +402,7 @@ def parseRoutine(inFile):
                     if m2.group("param"):
                         var += "(" + m2.group("param") + ")"
                     if m2.group("value"):
-                        var += " = "
+                        var += " {} ".format(m2["assignment"])
                         var += m2.group("value")
                     decl["vars"].append(var)
                     str = str[m2.span()[1] :]
@@ -368,7 +419,7 @@ def parseRoutine(inFile):
             elif INTERFACE_START_RE.match(jline):
                 istart = lines
                 interfaceDeclFile = StringIO()
-                while 1:
+                while True:
                     (jline, _, lines) = stream.nextFortranLine()
                     if INTERFACE_END_RE.match(jline):
                         iend = lines
@@ -376,8 +427,8 @@ def parseRoutine(inFile):
                     interfaceDeclFile.writelines(lines)
                 interfaceDeclFile = StringIO(interfaceDeclFile.getvalue())
                 iroutines = []
-                while 1:
-                    iroutine = parseRoutine(interfaceDeclFile)
+                while True:
+                    iroutine = parseRoutine(interfaceDeclFile, logger)
                     if not iroutine["kind"]:
                         if len(iroutines) == 0:
                             interfaceDeclFile.seek(0)
@@ -425,9 +476,9 @@ def parseRoutine(inFile):
         m = INCLUDE_RE.match(lines[0])
         if m:
             try:
-                subF = open(m.group("file"), "r")
+                subF = open(m.group("file"), "r", encoding="utf8")
                 subStream = InputStream(subF)
-                while 1:
+                while True:
                     (subjline, _, sublines) = subStream.nextFortranLine()
                     if not sublines:
                         break
@@ -436,12 +487,15 @@ def parseRoutine(inFile):
             except:
                 import traceback
 
-                logger.debug("error trying to follow include " + m.group("file") + "\n")
                 logger.debug(
-                    "warning this might lead to the removal of used variables\n"
+                    "error trying to follow include '{}', this might lead to the removal of used variables".format(
+                        m.group("file")
+                    )
                 )
+
                 if logger.isEnabledFor(logging.DEBUG):
                     traceback.print_exc()
+
         (jline, _, lines) = stream.nextFortranLine()
     return routine
 
@@ -689,16 +743,14 @@ def writeDeclarations(parsedDeclarations, file):
             writeExtendedDeclaration(d, file)
 
 
-def cleanDeclarations(routine):
+def cleanDeclarations(routine, logger):
     """cleans up the declaration part of the given parsed routine
     removes unused variables"""
-    logger = logging.getLogger("prettify-logger")
 
-    global R_VAR
     if routine["core"]:
         if CONTAINS_RE.match(routine["core"][-1]):
             logger.debug(
-                "routine %s contains other routines\ndeclarations not cleaned\n"
+                "routine %s contains other routines, declarations not cleaned"
                 % (routine["name"])
             )
             return
@@ -711,16 +763,15 @@ def cleanDeclarations(routine):
     if routine["core"]:
         if re.match(" *type *[a-zA-Z_]+ *$", routine["core"][0], re.IGNORECASE):
             logger.debug(
-                "routine %s contains local types, not fully cleaned\n"
-                % (routine["name"])
+                "routine %s contains local types, not fully cleaned" % (routine["name"])
             )
         if re.match(" *import+ *$", routine["core"][0], re.IGNORECASE):
             logger.debug(
-                "routine %s contains import, not fully cleaned\n" % (routine["name"])
+                "routine %s contains import, not fully cleaned" % (routine["name"])
             )
     if re.search("^#", "".join(routine["declarations"]), re.MULTILINE):
         logger.debug(
-            "routine %s declarations contain preprocessor directives\ndeclarations not cleaned\n"
+            "routine %s declarations contain preprocessor directives, declarations not cleaned"
             % (routine["name"])
         )
         return
@@ -809,7 +860,6 @@ def cleanDeclarations(routine):
                         logger.info(
                             "removed var %s in routine %s\n" % (lowerV, routine["name"])
                         )
-                        R_VAR += 1
             if len(localD["vars"]):
                 localDecl.append(localD)
         argDecl = []
@@ -818,12 +868,11 @@ def cleanDeclarations(routine):
                 argDecl.append(argDeclDict[arg])
             else:
                 logger.debug(
-                    "warning, implicitly typed argument '"
-                    + arg
-                    + "' in routine "
-                    + routine["name"]
-                    + "\n"
+                    "warning, implicitly typed argument '{}' in routine '{}'".format(
+                        arg, routine["name"]
+                    )
                 )
+
         if routine["kind"].lower() == "function":
             aDecl = argDecl[:-1]
         else:
@@ -852,8 +901,7 @@ def cleanDeclarations(routine):
 
         newDecl = StringIO()
         for comment in routine["preDeclComments"]:
-            if not COMMENT_TO_REMOVE_RE.match(comment):
-                newDecl.write(comment)
+            newDecl.write(comment)
         newDecl.writelines(routine["use"])
         writeDeclarations(argDecl, newDecl)
         if argDecl and paramDecl:
@@ -866,7 +914,7 @@ def cleanDeclarations(routine):
             newDecl.write("\n")
         wrote = 0
         for comment in routine["declComments"]:
-            if comment.strip() and not COMMENT_TO_REMOVE_RE.match(comment):
+            if comment.strip():
                 newDecl.write(comment.strip())
                 newDecl.write("\n")
                 wrote = 1
@@ -876,7 +924,7 @@ def cleanDeclarations(routine):
     except:
         if "name" in routine.keys():
             logger.critical("exception cleaning routine " + routine["name"])
-        logger.critical("parsedDeclartions=" + str(routine["parsedDeclarations"]))
+        logger.critical("parsedDeclartions={}".format(routine["parsedDeclarations"]))
         raise
 
     newDecl = StringIO()
@@ -889,9 +937,8 @@ def cleanDeclarations(routine):
                 comment_start += 1
 
         for comment in routine["postDeclComments"][comment_start:]:
-            if not COMMENT_TO_REMOVE_RE.match(comment):
-                newDecl.write(comment)
-                newDecl.write("\n")
+            newDecl.write(comment)
+            newDecl.write("\n")
         routine["declarations"][0] += newDecl.getvalue()
 
 
@@ -971,7 +1018,7 @@ def parseUse(inFile):
     origLines = []
     commonUses = ""
     stream = InputStream(inFile)
-    while 1:
+    while True:
         (jline, comment_list, lines) = stream.nextFortranLine()
         comments = "\n".join(_ for _ in comment_list if _)
         lineNr = lineNr + len(lines)
@@ -1106,7 +1153,7 @@ def writeUseShort(m, file):
 def prepareImplicitUses(modules):
     """Transforms a modulesDict into an implictUses (dictionary of module names
     each containing a dictionary with the only, and the special key '_WHOLE_'
-    wich is true if the whole mosule is implicitly present"""
+    which is true if the whole module is implicitly present"""
     mods = {}
     for m in modules:
         m_name = m["module"].lower()
@@ -1125,22 +1172,20 @@ def prepareImplicitUses(modules):
     return mods
 
 
-def cleanUse(modulesDict, rest, implicitUses=None):
-    """Removes the unneded modules (the ones that are not used in rest)"""
-    logger = logging.getLogger("prettify-logger")
+def cleanUse(modulesDict, rest, implicitUses, logger):
+    """Removes the unneeded modules (the ones that are not used in rest)"""
 
-    global R_USE
     exceptions = {}
     modules = modulesDict["modules"]
     rest = rest.lower()
+
     for i in range(len(modules) - 1, -1, -1):
         m_att = {}
         m_name = modules[i]["module"].lower()
         if implicitUses and m_name in implicitUses.keys():
             m_att = implicitUses[m_name]
         if "_WHOLE_" in m_att.keys() and m_att["_WHOLE_"]:
-            R_USE += 1
-            logger.info("removed USE of module " + m_name + "\n")
+            logger.info("removed USE of module " + m_name)
             del modules[i]
         elif "only" in modules[i].keys():
             els = modules[i]["only"]
@@ -1150,17 +1195,11 @@ def cleanUse(modulesDict, rest, implicitUses=None):
                     raise SyntaxError("could not parse use only:" + repr(els[j]))
                 impAtt = m.group("localName").lower()
                 if impAtt in m_att.keys():
-                    R_USE += 1
-                    logger.info(
-                        "removed USE " + m_name + ", only: " + repr(els[j]) + "\n"
-                    )
+                    logger.info("removed USE " + m_name + ", only: " + repr(els[j]))
                     del els[j]
                 elif impAtt not in exceptions.keys():
                     if findWord(impAtt, rest) == -1:
-                        R_USE += 1
-                        logger.info(
-                            "removed USE " + m_name + ", only: " + repr(els[j]) + "\n"
-                        )
+                        logger.info("removed USE " + m_name + ", only: " + repr(els[j]))
                         del els[j]
             if len(modules[i]["only"]) == 0:
                 if modules[i]["comments"]:
@@ -1189,8 +1228,6 @@ def rewriteFortranFile(
     It sorts them and removes the repetitions."""
     import os.path
 
-    logger = logging.getLogger("prettify-logger")
-
     global INDENT_SIZE
     global DECL_OFFSET
     global DECL_LINELENGTH
@@ -1199,7 +1236,7 @@ def rewriteFortranFile(
     DECL_LINELENGTH = decl_linelength
 
     coreLines = []
-    while 1:
+    while True:
         line = inFile.readline()
         if not line:
             break
@@ -1213,68 +1250,67 @@ def rewriteFortranFile(
             fn = os.path.basename(orig_filename).rsplit(".", 1)[0]
             break
 
-    try:
-        modulesDict = parseUse(inFile)
-        routines = []
-        coreLines.append(modulesDict["postLine"])
-        routine = parseRoutine(inFile)
-        coreLines.extend(routine["preRoutine"])
-        if m:
-            resetModuleN(m.group("moduleName"), routine["preRoutine"])
+    logger = logging.LoggerAdapter(
+        logging.getLogger("prettify-logger"), {"ffilename": orig_filename}
+    )
+
+    modulesDict = parseUse(inFile)
+    routines = []
+    coreLines.append(modulesDict["postLine"])
+    routine = parseRoutine(inFile, logger)
+    coreLines.extend(routine["preRoutine"])
+
+    if m:
+        resetModuleN(m.group("moduleName"), routine["preRoutine"])
+
+    routines.append(routine)
+
+    while routine["kind"]:
+        routine = parseRoutine(inFile, logger)
         routines.append(routine)
-        while routine["kind"]:
-            routine = parseRoutine(inFile)
-            routines.append(routine)
-        for routine in routines:
-            cleanDeclarations(routine)  # in-place modification of 'routine'
-            coreLines.extend(routine["declarations"])
-            coreLines.extend(routine["strippedCore"])
-        rest = "".join(coreLines)
-        nonStPrep = 0
-        for line in modulesDict["origLines"]:
-            if re.search("^#", line) and not COMMON_USES_RE.match(line):
-                logger.debug("noMatch " + repr(line) + "\n")  # what does it mean?
-                nonStPrep = 1
-        if nonStPrep:
-            logger.debug(
-                "use statements contains preprocessor directives, not cleaning\n"
-            )
-            outFile.writelines(modulesDict["origLines"])
-        else:
-            implicitUses = None
-            if modulesDict["commonUses"]:
-                try:
-                    inc_fn = COMMON_USES_RE.match(modulesDict["commonUses"]).group(1)
-                    inc_absfn = os.path.join(os.path.dirname(orig_filename), inc_fn)
-                    f = open(inc_absfn, "r")
-                    implicitUsesRaw = parseUse(f)
-                    f.close()
-                    implicitUses = prepareImplicitUses(implicitUsesRaw["modules"])
-                except:
-                    logger.critical(
-                        "ERROR trying to parse use statements contained in common uses precompiler file "
-                        + inc_absfn
-                        + "\n"
+
+    for routine in routines:
+        cleanDeclarations(routine, logger)  # in-place modification of 'routine'
+        coreLines.extend(routine["declarations"])
+        coreLines.extend(routine["strippedCore"])
+
+    rest = "".join(coreLines)
+    nonStPrep = 0
+
+    for line in modulesDict["origLines"]:
+        if re.search("^#", line) and not COMMON_USES_RE.match(line):
+            logger.debug("noMatch " + repr(line) + "\n")  # what does it mean?
+            nonStPrep = 1
+
+    if nonStPrep:
+        logger.debug("use statements contains preprocessor directives, not cleaning")
+        outFile.writelines(modulesDict["origLines"])
+    else:
+        implicitUses = None
+        if modulesDict["commonUses"]:
+            inc_fn = COMMON_USES_RE.match(modulesDict["commonUses"]).group(1)
+            inc_absfn = os.path.join(os.path.dirname(orig_filename), inc_fn)
+            try:
+                with open(inc_absfn, "r", encoding="utf8") as fhandle:
+                    implicitUsesRaw = parseUse(fhandle)
+                implicitUses = prepareImplicitUses(implicitUsesRaw["modules"])
+            except:
+                logger.critical(
+                    "failed to parse use statements contained in common uses precompiler file {}".format(
+                        inc_absfn
                     )
-                    raise
-            cleanUse(modulesDict, rest, implicitUses=implicitUses)
-            normalizeModules(modulesDict["modules"])
-            outFile.writelines(modulesDict["preComments"])
-            writeUses(modulesDict["modules"], outFile)
-            outFile.write(modulesDict["commonUses"])
-            if modulesDict["modules"]:
-                outFile.write("\n")
-        outFile.write(modulesDict["postLine"])
-        for routine in routines:
-            writeRoutine(routine, outFile)
-    except:
-        import traceback
+                )
+                raise
 
-        logger.critical("-" * 60 + "\n")
-        traceback.print_exc(file=sys.stderr)
-        logger.critical("-" * 60 + "\n")
-        logger.critical("Processing file '" + orig_filename + "'\n")
-        raise
+        cleanUse(modulesDict, rest, implicitUses, logger)
+        normalizeModules(modulesDict["modules"])
+        outFile.writelines(modulesDict["preComments"])
+        writeUses(modulesDict["modules"], outFile)
+        outFile.write(modulesDict["commonUses"])
+        if modulesDict["modules"]:
+            outFile.write("\n")
 
+    outFile.write(modulesDict["postLine"])
 
-# EOF
+    for routine in routines:
+        writeRoutine(routine, outFile)
