@@ -1,5 +1,12 @@
 # Common variables used by the installation scripts
 
+# TODO: Review and if possible fix shellcheck errors.
+# shellcheck disable=SC1003,SC1035,SC1083,SC1090
+# shellcheck disable=SC2001,SC2002,SC2005,SC2016,SC2091,SC2034,SC2046,SC2086,SC2089,SC2090
+# shellcheck disable=SC2124,SC2129,SC2144,SC2153,SC2154,SC2155,SC2163,SC2164,SC2166
+# shellcheck disable=SC2235,SC2237
+# shellcheck shell=bash
+
 # directories and files used by the installer
 ROOTDIR=${ROOTDIR:-"$(pwd -P)"}
 SCRIPTDIR=${SCRIPTDIR:-"${ROOTDIR}/scripts"}
@@ -22,17 +29,13 @@ SYS_LIB_PATH=${SYS_LIB_PATHS:-'/usr/local/lib64:/usr/local/lib:/usr/lib64:/usr/l
 INCLUDE_PATHS=${INCLUDE_PATHS:-"CPATH SYS_INCLUDE_PATH"}
 LIB_PATHS=${LIB_PATHS:-'LD_LIBRARY_PATH LIBRARY_PATH LD_RUN_PATH SYS_LIB_PATH'}
 
-# number of processors
-NPROCS=${NPROCS:-1}
-
 # mode flags
 ENABLE_OMP=${ENABLE_OMP:-"__TRUE__"}
-ENABLE_TSAN=${ENABLE_TSAN:-"__FALSE__"}
-ENABLE_VALGRIND=${ENABLE_VALGRIND:-"__FALSE__"}
 ENABLE_CUDA=${ENABLE_CUDA:-"__FALSE__"}
+ENABLE_HIP=${ENABLE_HIP:-"__FALSE__"}
 ENABLE_CRAY=${ENABLE_CRAY:-"__FALSE__"}
 MPI_MODE=${MPI_MODE:-openmpi}
-FAST_MATH_MODE=${FAST_MATH_MODE:-openblas}
+MATH_MODE=${MATH_MODE:-openblas}
 
 # compiler flags
 export CC=${CC:-gcc}
